from tuikit.window import Window
from tuikit.layout import OffsetLayout, AnchorLayout
from tuikit.scrollbar import VScrollbar, HScrollbar
from tuikit.common import Borders


class ScrollWindow(Window):
    def __init__(self):
        Window.__init__(self, inner_layout=OffsetLayout)

        self.vscroll = VScrollbar()
        self.vscroll.add_handler('change', self._on_vscroll_change)
        AnchorLayout.add(self, self.vscroll, halign='right', valign='fill', margin=Borders(t=1, b=1))

        self.hscroll = HScrollbar()
        self.hscroll.add_handler('change', self._on_hscroll_change)
        AnchorLayout.add(self, self.hscroll, halign='fill', valign='bottom', margin=Borders(l=1, r=2))

    def add(self, widget, **kwargs):
        Window.add(self, widget, **kwargs)
        widget.add_handler('sizereq', self._on_child_sizereq)
        widget.add_handler('spotmove', self._on_child_spotmove)
        widget.add_handler('scrollreq', self._on_child_scrollreq)

    def on_resize(self, ev):
        self._update_scroll_max()

    def _on_child_sizereq(self, ev):
        self._update_scroll_max()

    def _on_vscroll_change(self, ev):
        self._inner.offset.y = - self.vscroll.scroll_pos

    def _on_hscroll_change(self, ev):
        self._inner.offset.x = - self.hscroll.scroll_pos

    def _on_child_spotmove(self, ev):
        child = ev.originator
        # x
        spot_x = child.x - self._inner.offset.x + child.spot.x
        if spot_x < self.hscroll.scroll_pos:
            self.hscroll.scroll_pos = spot_x
        if spot_x > (self._inner.width - 1) + self.hscroll.scroll_pos:
            self.hscroll.scroll_pos = spot_x - (self._inner.width - 1)
        # y
        spot_y = child.y - self._inner.offset.y + child.spot.y
        if spot_y < self.vscroll.scroll_pos:
            self.vscroll.scroll_pos = spot_y
        if spot_y > (self._inner.height - 1) + self.vscroll.scroll_pos:
            self.vscroll.scroll_pos = spot_y - (self._inner.height - 1)

    def _on_child_scrollreq(self, ev):
        new_scroll_pos = self.vscroll.scroll_pos + ev.data
        if new_scroll_pos > self.vscroll.scroll_max:
            self.vscroll.scroll_pos = self.vscroll.scroll_max
        elif new_scroll_pos < 0:
            self.vscroll.scroll_pos = 0
        else:
            self.vscroll.scroll_pos = new_scroll_pos

    def _update_scroll_max(self):
        max_width = 0
        max_height = 0
        for child in self._inner.children:
            child_width = child.x - self._inner.offset.x + child.sizereq.w
            if child_width > max_width:
                max_width = child_width
            child_height = child.y - self._inner.offset.y + child.sizereq.h
            if child_height > max_height:
                max_height = child_height
        max_width += 1
        if max_width < self._inner.width:
            self.hscroll.hide()
        else:
            self.hscroll.scroll_max = max_width - self._inner.width
            self.hscroll.show()
        if max_height < self._inner.height:
            self.vscroll.hide()
        else:
            self.vscroll.scroll_max = max_height - self._inner.height
            self.vscroll.show()

